Package: micEcon
Version: 0.6-18
Date: 2022-09-03
Title: Microeconomic Analysis and Modelling
Author: Arne Henningsen
Maintainer: Arne Henningsen <arne.henningsen@gmail.com>
Depends: R (>= 3.5.0)
Imports: miscTools (>= 0.6-1), plm (>= 1.1-0)
Suggests: Ecdat (>= 0.1-5), systemfit (>= 1.0-0)
Description: Various tools for microeconomic analysis and microeconomic modelling,
   e.g. estimating quadratic, Cobb-Douglas and Translog functions,
   calculating partial derivatives and elasticities of these functions,
   and calculating Hessian matrices, checking curvature
   and preparing restrictions for imposing monotonicity of Translog functions.
License: GPL (>= 2)
URL: http://www.micEcon.org
NeedsCompilation: no
Packaged: 2022-09-03 20:03:18 UTC; gsl324
Repository: CRAN
Date/Publication: 2022-09-03 21:30:02 UTC
Built: R 4.4.0; ; 2024-04-29 02:42:36 UTC; unix
