export( "checkConsist" )
export( "cobbDouglasCalc" )
export( "cobbDouglasDeriv" )
export( "cobbDouglasOpt" )
export( "elas" )
export( "elasticities" )
export( "logDataSet" )
export( "quadFuncCalc" )
export( "quadFuncDeriv" )
export( "quadFuncEla" )
export( "quadFuncEst" )
export( "translogCalc" )
export( "translogCheckCurvature" )
export( "translogCheckMono" )
export( "translogCostEst" )
export( "translogDeriv" )
export( "translogEla" )
export( "translogEst" )
export( "translogHessian" )
export( "translogMonoRestr" )
export( "translogProdFuncMargCost" )
export( ".quadFuncCheckHomWeights" )
export( ".quadFuncVarHom" )

S3method( "coef", "quadFuncEst" )
S3method( "coef", "translogEst" )
S3method( "elas", "default" )
S3method( "elas", "quadFuncEst" )
S3method( "elas", "translogEst" )
S3method( "predict", "quadFuncEst" )
S3method( "predict", "translogEst" )
S3method( "print", "summary.translogCheckMono" )
S3method( "print", "summary.translogEst" )
S3method( "print", "translogCheckCurvature" )
S3method( "print", "translogCheckMono" )
S3method( "print", "translogEst" )
S3method( "residuals", "translogEst" )
S3method( "summary", "translogCheckMono" )
S3method( "summary", "translogEst" )
S3method( "vcov", "quadFuncEst" )
S3method( "vcov", "translogEst" )

importFrom( "miscTools", "checkNames" )
importFrom( "miscTools", "coefTable" )
importFrom( "miscTools", "rSquared" )
importFrom( "miscTools", "quasiconcavity" )
importFrom( "miscTools", "quasiconvexity" )
importFrom( "miscTools", "semidefiniteness" )
importFrom( "miscTools", "vecli" )
importFrom( "miscTools", "vecli2m" )
importFrom( "miscTools", "veclipos" )

importFrom( "plm", "index" )
importFrom( "plm", "pdata.frame" )
importFrom( "plm", "plm" )
importFrom( "plm", "fixef" )

importFrom( "stats", "as.formula" )
importFrom( "stats", "coef" )
importFrom( "stats", "df.residual" )
importFrom( "stats", "lm" )
importFrom( "stats", "model.matrix" )
importFrom( "stats", "predict" )
importFrom( "stats", "printCoefmat" )
importFrom( "stats", "residuals" )
importFrom( "stats", "vcov" )
